/*
** comp.c -- compose a message
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <h/mh.h>
#include <h/utils.h>
#include <fcntl.h>

static struct swit switches[] = {
#define EDITRSW  0
	{ "editor editor", 0 },
#define NEDITSW  1
	{ "noedit", 0 },
#define FORMSW  2
	{ "form formfile", 0 },
#define USESW  3
	{ "use", 0 },
#define NUSESW  4
	{ "nouse", 0 },
#define WHATSW  5
	{ "whatnowproc program", 0 },
#define NWHATSW  6
	{ "nowhatnowproc", 0 },
#define VERSIONSW  7
	{ "version", 0 },
#define HELPSW  8
	{ "help", 0 },
	{ NULL, 0 }
};


int
main(int argc, char **argv)
{
	int use = NOUSE, nedit = 0, nwhat = 0;
	int in, out;
	char *cp, *cwd, *maildir;
	char *ed = NULL, *form = NULL;
	char *folder = NULL, *msg = NULL, buf[BUFSIZ];
	char drft[BUFSIZ], **argp, **arguments;
	struct msgs *mp = NULL;
	char *fmtstr;


#ifdef LOCALE
	setlocale(LC_ALL, "");
#endif
	invo_name = mhbasename(argv[0]);

	/* read user profile/context */
	context_read();

	arguments = getarguments(invo_name, argc, argv, 1);
	argp = arguments;

	while ((cp = *argp++)) {
		if (*cp == '-') {
			switch (smatch(++cp, switches)) {
			case AMBIGSW:
				ambigsw(cp, switches);
				done(1);
			case UNKWNSW:
				adios(NULL, "-%s unknown", cp);

			case HELPSW:
				snprintf(buf, sizeof(buf), "%s [+folder] [msg] [switches]", invo_name);
				print_help(buf, switches, 1);
				done(1);
			case VERSIONSW:
				print_version(invo_name);
				done(1);

			case EDITRSW:
				if (!(ed = *argp++) || *ed == '-')
					adios(NULL, "missing argument to %s", argp[-2]);
				nedit = 0;
				continue;
			case NEDITSW:
				nedit++;
				continue;

			case WHATSW:
				if (!(whatnowproc = *argp++) || *whatnowproc == '-')
					adios(NULL, "missing argument to %s", argp[-2]);
				nwhat = 0;
				continue;
			case NWHATSW:
				nwhat++;
				continue;

			case FORMSW:
				if (!(form = *argp++) || *form == '-')
					adios(NULL, "missing argument to %s", argp[-2]);
				continue;

			case USESW:
				use++;
				continue;
			case NUSESW:
				use = NOUSE;
				continue;
			}
		}
		if (*cp == '+' || *cp == '@') {
			if (folder)
				adios(NULL, "only one folder at a time!");
			else
				folder = getcpy(expandfol(cp));
		} else {
			if (msg)
				adios(NULL, "only one message at a time!");
			else
				msg = cp;
		}
	}

	cwd = getcpy(pwd());

	if (form && (folder || msg))
		adios(NULL, "can't mix forms and folders/msgs");

	if (use && folder)
		adios(NULL, "can't mix -use and +folder");

	if (use) {
		/* Don't copy; the draft shall get removed in the end. */
		strncpy(drft, m_draft(msg ? msg : seq_cur), sizeof(drft));

	} else if (folder || msg) {
		/* Take a message as the "form" for the new message. */
		if (!msg)
			msg = seq_cur;
		if (!folder)
			folder = getcurfol();
		maildir = toabsdir(folder);
		if (chdir(maildir) == NOTOK)
			adios(maildir, "unable to change directory to");
		/* read folder and create message structure */
		if (!(mp = folder_read(folder)))
			adios(NULL, "unable to read folder %s", folder);
		/* check for empty folder */
		if (mp->nummsg == 0)
			adios(NULL, "no messages in %s", folder);
		/* parse the message range/sequence/name and set SELECTED */
		if (!m_convert(mp, msg))
			done(1);
		seq_setprev(mp);  /* set the previous-sequence */
		if (mp->numsel > 1)
			adios(NULL, "only one message at a time!");
		if ((in = open(form = getcpy(m_name(mp->lowsel)),
				O_RDONLY)) == NOTOK)
			adios(form, "unable to open message");

		strncpy(drft, m_draft(seq_beyond), sizeof(drft));
		if ((out = creat(drft, m_gmprot())) == NOTOK) {
			adios(drft, "unable to create");
		}
		cpydata(in, out, form, drft);
		close(in);
		close(out);

	} else {
		fmtstr = new_fs(form, components);
		strncpy(drft, m_draft(seq_beyond), sizeof(drft));
		if ((out = creat(drft, m_gmprot())) == NOTOK) {
			adios(drft, "unable to create");
		}
		if (write(out, fmtstr, strlen(fmtstr)) != (int)strlen(fmtstr)) {
			adios(drft, "error writing");
		}
		close(out);
	}

	context_save();

	if (nwhat) {
		done(0);
	}
	what_now(ed, nedit, use, drft, NULL, 0, NULLMP, NULL, cwd);
	done(1);
	return 1;
}
